@@ -1,12 +1,12 @@
 """Default pipeline."""
-from collections.abc import Iterable
+from collections.abc import Iterator
 from functools import partial
 from pathlib import Path
-import tempfile
-from typing import final
-from urllib.parse import quote_plus
+from tempfile import gettempdir
+from typing import TYPE_CHECKING, final
 
-from jaxtyping import Int, Int64
+from jaxtyping import Float, Int, Int64
+from pydantic import NonNegativeInt, PositiveInt, validate_call
 import torch
 from torch import Tensor
 from torch.utils.data import DataLoader
@@ -20,7 +20,7 @@ from sparse_autoencoder.activation_resampler.abstract_activation_resampler impor
 )
 from sparse_autoencoder.activation_store.tensor_store import TensorActivationStore
 from sparse_autoencoder.autoencoder.model import SparseAutoencoder
-from sparse_autoencoder.loss.abstract_loss import AbstractLoss
+from sparse_autoencoder.loss.abstract_loss import AbstractLoss, LossReductionType
 from sparse_autoencoder.metrics.metrics_container import MetricsContainer, default_metrics
 from sparse_autoencoder.metrics.train.abstract_train_metric import TrainMetricData
 from sparse_autoencoder.metrics.validate.abstract_validate_metric import ValidationMetricData
@@ -30,10 +30,13 @@ from sparse_autoencoder.source_model.replace_activations_hook import replace_act
 from sparse_autoencoder.source_model.store_activations_hook import store_activations_hook
 from sparse_autoencoder.source_model.zero_ablate_hook import zero_ablate_hook
 from sparse_autoencoder.tensor_types import Axis
-from sparse_autoencoder.train.utils import get_model_device
+from sparse_autoencoder.train.utils.get_model_device import get_model_device
 
 
-DEFAULT_CHECKPOINT_DIRECTORY: Path = Path(tempfile.gettempdir()) / "sparse_autoencoder"
+if TYPE_CHECKING:
+    from sparse_autoencoder.metrics.abstract_metric import MetricResult
+
+DEFAULT_CHECKPOINT_DIRECTORY: Path = Path(gettempdir()) / "sparse_autoencoder"
 
 
 class Pipeline:
@@ -49,11 +52,11 @@ class Pipeline:
     autoencoder: SparseAutoencoder
     """Sparse autoencoder to train."""
 
-    cache_name: str
-    """Name of the cache to use in the source model (hook point)."""
+    cache_names: list[str]
+    """Names of the cache hook points to use in the source model."""
 
     layer: int
-    """Layer to get activations from with the source model."""
+    """Layer to stope the source model at (if we don't need activations after this layer)."""
 
     log_frequency: int
     """Frequency at which to log metrics (in steps)."""
@@ -70,7 +73,7 @@ class Pipeline:
     progress_bar: tqdm | None
     """Progress bar for the pipeline."""
 
-    source_data: Iterable[TorchTokenizedPrompts]
+    source_data: Iterator[TorchTokenizedPrompts]
     """Iterable over the source data."""
 
     source_dataset: SourceDataset
@@ -82,30 +85,37 @@ class Pipeline:
     total_activations_trained_on: int = 0
     """Total number of activations trained on state."""
 
+    @property
+    def n_components(self) -> int:
+        """Number of source model components the SAE is trained on."""
+        return len(self.cache_names)
+
     @final
+    @validate_call(config={"arbitrary_types_allowed": True})
     def __init__(
         self,
         activation_resampler: AbstractActivationResampler | None,
         autoencoder: SparseAutoencoder,
-        cache_name: str,
-        layer: int,
+        cache_names: list[str],
+        layer: NonNegativeInt,
         loss: AbstractLoss,
         optimizer: AbstractOptimizerWithReset,
         source_dataset: SourceDataset,
         source_model: HookedTransformer,
         run_name: str = "sparse_autoencoder",
         checkpoint_directory: Path = DEFAULT_CHECKPOINT_DIRECTORY,
-        log_frequency: int = 100,
+        log_frequency: PositiveInt = 100,
         metrics: MetricsContainer = default_metrics,
-        source_data_batch_size: int = 12,
+        source_data_batch_size: PositiveInt = 12,
     ) -> None:
         """Initialize the pipeline.
 
         Args:
             activation_resampler: Activation resampler to use.
             autoencoder: Sparse autoencoder to train.
-            cache_name: Name of the cache to use in the source model (hook point).
-            layer: Layer to get activations from with the source model.
+            cache_names: Names of the cache hook points to use in the source model.
+            layer: Layer to stope the source model at (if we don't need activations after this
+                layer).
             loss: Loss function to use.
             optimizer: Optimizer to use.
             source_dataset: Source dataset to get data from.
@@ -118,7 +128,7 @@ class Pipeline:
         """
         self.activation_resampler = activation_resampler
         self.autoencoder = autoencoder
-        self.cache_name = cache_name
+        self.cache_names = cache_names
         self.checkpoint_directory = checkpoint_directory
         self.layer = layer
         self.log_frequency = log_frequency
@@ -130,10 +140,12 @@ class Pipeline:
         self.source_dataset = source_dataset
         self.source_model = source_model
 
+        # Create a stateful iterator
         source_dataloader = source_dataset.get_dataloader(source_data_batch_size)
-        self.source_data = self.stateful_dataloader_iterable(source_dataloader)
+        self.source_data = iter(source_dataloader)
 
-    def generate_activations(self, store_size: int) -> TensorActivationStore:
+    @validate_call
+    def generate_activations(self, store_size: PositiveInt) -> TensorActivationStore:
         """Generate activations.
 
         Args:
@@ -143,13 +155,10 @@ class Pipeline:
             Activation store for the train section.
 
         Raises:
-            ValueError: If the store size is not positive or is not divisible by the batch size.
+            ValueError: If the store size is not divisible by the batch size.
         """
-        # Check the store size is positive and divisible by the batch size
-        if store_size <= 0:
-            error_message = f"Store size must be positive, got {store_size}"
-            raise ValueError(error_message)
-        if store_size % self.source_data_batch_size != 0:
+        # Check the store size is divisible by the batch size
+        if store_size % (self.source_data_batch_size * self.source_dataset.context_size) != 0:
             error_message = (
                 f"Store size must be divisible by the batch size ({self.source_data_batch_size}), "
                 f"got {store_size}"
@@ -157,19 +166,21 @@ class Pipeline:
             raise ValueError(error_message)
 
         # Setup the store
-        num_neurons: int = self.autoencoder.n_input_features
+        n_neurons: int = self.autoencoder.config.n_input_features
         source_model_device: torch.device = get_model_device(self.source_model)
-        store = TensorActivationStore(store_size, num_neurons)
+        store = TensorActivationStore(store_size, n_neurons, n_components=self.n_components)
 
         # Add the hook to the model (will automatically store the activations every time the model
         # runs)
         self.source_model.remove_all_hook_fns()
-        hook = partial(store_activations_hook, store=store)
-        self.source_model.add_hook(self.cache_name, hook)
+        for component_idx, cache_name in enumerate(self.cache_names):
+            hook = partial(store_activations_hook, store=store, component_idx=component_idx)
+            self.source_model.add_hook(cache_name, hook)
 
         # Loop through the dataloader until the store reaches the desired size
         with torch.no_grad():
-            for batch in self.source_data:
+            while len(store) < store_size:
+                batch = next(self.source_data)
                 input_ids: Int[Tensor, Axis.names(Axis.SOURCE_DATA_BATCH, Axis.POSITION)] = batch[
                     "input_ids"
                 ].to(source_model_device)
@@ -177,17 +188,15 @@ class Pipeline:
                     input_ids, stop_at_layer=self.layer + 1, prepend_bos=False
                 )  # type: ignore (TLens is typed incorrectly)
 
-                if len(store) >= store_size:
-                    break
-
         self.source_model.remove_all_hook_fns()
         store.shuffle()
 
         return store
 
+    @validate_call(config={"arbitrary_types_allowed": True})
     def train_autoencoder(
-        self, activation_store: TensorActivationStore, train_batch_size: int
-    ) -> Int64[Tensor, Axis.LEARNT_FEATURE]:
+        self, activation_store: TensorActivationStore, train_batch_size: PositiveInt
+    ) -> Int64[Tensor, Axis.names(Axis.COMPONENT, Axis.LEARNT_FEATURE)]:
         """Train the sparse autoencoder.
 
         Args:
@@ -195,7 +204,7 @@ class Pipeline:
             train_batch_size: Train batch size.
 
         Returns:
-            Number of times each neuron fired.
+            Number of times each neuron fired, for each component.
         """
         autoencoder_device: torch.device = get_model_device(self.autoencoder)
 
@@ -204,8 +213,12 @@ class Pipeline:
             batch_size=train_batch_size,
         )
 
-        learned_activations_fired_count: Int64[Tensor, Axis.LEARNT_FEATURE] = torch.zeros(
-            self.autoencoder.n_learned_features, dtype=torch.int64, device=autoencoder_device
+        learned_activations_fired_count: Int64[
+            Tensor, Axis.names(Axis.COMPONENT, Axis.LEARNT_FEATURE)
+        ] = torch.zeros(
+            (self.n_components, self.autoencoder.config.n_learned_features),
+            dtype=torch.int64,
+            device=autoencoder_device,
         )
 
         for store_batch in activations_dataloader:
@@ -216,21 +229,24 @@ class Pipeline:
             batch = store_batch.detach().to(autoencoder_device)
 
             # Forward pass
-            learned_activations, reconstructed_activations = self.autoencoder(batch)
+            learned_activations, reconstructed_activations = self.autoencoder.forward(batch)
 
             # Get loss & metrics
-            metrics = {}
-            total_loss, loss_metrics = self.loss.batch_scalar_loss_with_log(
-                batch, learned_activations, reconstructed_activations
+            metrics: list[MetricResult] = []
+            total_loss, loss_metrics = self.loss.scalar_loss_with_log(
+                batch,
+                learned_activations,
+                reconstructed_activations,
+                component_reduction=LossReductionType.MEAN,
             )
-            metrics.update(loss_metrics)
+            metrics.extend(loss_metrics)
 
             with torch.no_grad():
                 for metric in self.metrics.train_metrics:
                     calculated = metric.calculate(
                         TrainMetricData(batch, learned_activations, reconstructed_activations)
                     )
-                    metrics.update(calculated)
+                    metrics.extend(calculated)
 
             # Store count of how many neurons have fired
             with torch.no_grad():
@@ -240,7 +256,7 @@ class Pipeline:
             # Backwards pass
             total_loss.backward()
             self.optimizer.step()
-            self.autoencoder.decoder.constrain_weights_unit_norm()
+            self.autoencoder.post_backwards_hook()
 
             # Log training metrics
             self.total_activations_trained_on += train_batch_size
@@ -249,98 +265,128 @@ class Pipeline:
                 and int(self.total_activations_trained_on / train_batch_size) % self.log_frequency
                 == 0
             ):
+                log = {}
+                for metric_result in metrics:
+                    log.update(metric_result.wandb_log)
                 wandb.log(
-                    data={**metrics, **loss_metrics},
+                    log,
                     step=self.total_activations_trained_on,
                     commit=True,
                 )
 
         return learned_activations_fired_count
 
-    def update_parameters(self, parameter_updates: ParameterUpdateResults) -> None:
+    def update_parameters(self, parameter_updates: list[ParameterUpdateResults]) -> None:
         """Update the parameters of the model from the results of the resampler.
 
         Args:
             parameter_updates: Parameter updates from the resampler.
         """
-        # Update the weights and biases
-        self.autoencoder.encoder.update_dictionary_vectors(
-            parameter_updates.dead_neuron_indices,
-            parameter_updates.dead_encoder_weight_updates,
-        )
-        self.autoencoder.encoder.update_bias(
-            parameter_updates.dead_neuron_indices,
-            parameter_updates.dead_encoder_bias_updates,
-        )
-        self.autoencoder.decoder.update_dictionary_vectors(
-            parameter_updates.dead_neuron_indices,
-            parameter_updates.dead_decoder_weight_updates,
-        )
-
-        # Reset the optimizer
-        for parameter, axis in self.autoencoder.reset_optimizer_parameter_details:
-            self.optimizer.reset_neurons_state(
-                parameter=parameter,
-                neuron_indices=parameter_updates.dead_neuron_indices,
-                axis=axis,
+        for component_idx, component_parameter_update in enumerate(parameter_updates):
+            # Update the weights and biases
+            self.autoencoder.encoder.update_dictionary_vectors(
+                component_parameter_update.dead_neuron_indices,
+                component_parameter_update.dead_encoder_weight_updates,
+                component_idx=component_idx,
+            )
+            self.autoencoder.encoder.update_bias(
+                component_parameter_update.dead_neuron_indices,
+                component_parameter_update.dead_encoder_bias_updates,
+                component_idx=component_idx,
+            )
+            self.autoencoder.decoder.update_dictionary_vectors(
+                component_parameter_update.dead_neuron_indices,
+                component_parameter_update.dead_decoder_weight_updates,
+                component_idx=component_idx,
             )
 
-    def validate_sae(self, validation_number_activations: int) -> None:
+            # Reset the optimizer
+            for parameter, axis in self.autoencoder.reset_optimizer_parameter_details:
+                self.optimizer.reset_neurons_state(
+                    parameter=parameter,
+                    neuron_indices=component_parameter_update.dead_neuron_indices,
+                    axis=axis,
+                    component_idx=component_idx,
+                )
+
+    @validate_call
+    def validate_sae(self, validation_n_activations: PositiveInt) -> None:
         """Get validation metrics.
 
         Args:
-            validation_number_activations: Number of activations to use for validation.
+            validation_n_activations: Number of activations to use for validation.
         """
-        losses: list[float] = []
-        losses_with_reconstruction: list[float] = []
-        losses_with_zero_ablation: list[float] = []
+        losses_shape = (
+            validation_n_activations // self.source_data_batch_size,
+            self.n_components,
+        )
         source_model_device: torch.device = get_model_device(self.source_model)
 
-        for batch in self.source_data:
-            input_ids: Int[Tensor, Axis.names(Axis.SOURCE_DATA_BATCH, Axis.POSITION)] = batch[
-                "input_ids"
-            ].to(source_model_device)
+        # Create the metric data stores
+        losses: Float[Tensor, Axis.names(Axis.ITEMS, Axis.COMPONENT)] = torch.empty(
+            losses_shape, device=source_model_device
+        )
+        losses_with_reconstruction: Float[
+            Tensor, Axis.names(Axis.ITEMS, Axis.COMPONENT)
+        ] = torch.empty(losses_shape, device=source_model_device)
+        losses_with_zero_ablation: Float[
+            Tensor, Axis.names(Axis.ITEMS, Axis.COMPONENT)
+        ] = torch.empty(losses_shape, device=source_model_device)
 
-            # Run a forward pass with and without the replaced activations
-            self.source_model.remove_all_hook_fns()
-            replacement_hook = partial(
-                replace_activations_hook, sparse_autoencoder=self.autoencoder
-            )
+        for component_idx, cache_name in enumerate(self.cache_names):
+            for batch_idx in range(losses.shape[0]):
+                batch = next(self.source_data)
 
-            loss = self.source_model.forward(input_ids, return_type="loss")
-            loss_with_reconstruction = self.source_model.run_with_hooks(
-                input_ids,
-                return_type="loss",
-                fwd_hooks=[
-                    (
-                        self.cache_name,
-                        replacement_hook,
+                input_ids: Int[Tensor, Axis.names(Axis.SOURCE_DATA_BATCH, Axis.POSITION)] = batch[
+                    "input_ids"
+                ].to(source_model_device)
+
+                # Run a forward pass with and without the replaced activations
+                self.source_model.remove_all_hook_fns()
+                replacement_hook = partial(
+                    replace_activations_hook,
+                    sparse_autoencoder=self.autoencoder,
+                    component_idx=component_idx,
+                )
+
+                with torch.no_grad():
+                    loss = self.source_model.forward(input_ids, return_type="loss")
+                    loss_with_reconstruction = self.source_model.run_with_hooks(
+                        input_ids,
+                        return_type="loss",
+                        fwd_hooks=[
+                            (
+                                cache_name,
+                                replacement_hook,
+                            )
+                        ],
+                    )
+                    loss_with_zero_ablation = self.source_model.run_with_hooks(
+                        input_ids,
+                        return_type="loss",
+                        fwd_hooks=[(cache_name, zero_ablate_hook)],
                     )
-                ],
-            )
-            loss_with_zero_ablation = self.source_model.run_with_hooks(
-                input_ids,
-                return_type="loss",
-                fwd_hooks=[(self.cache_name, zero_ablate_hook)],
-            )
 
-            losses.append(loss.sum().item())
-            losses_with_reconstruction.append(loss_with_reconstruction.sum().item())
-            losses_with_zero_ablation.append(loss_with_zero_ablation.sum().item())
-
-            if len(losses) >= validation_number_activations // input_ids.numel():
-                break
+                    losses[batch_idx, component_idx] = loss.sum()
+                    losses_with_reconstruction[
+                        batch_idx, component_idx
+                    ] = loss_with_reconstruction.sum()
+                    losses_with_zero_ablation[
+                        batch_idx, component_idx
+                    ] = loss_with_zero_ablation.sum()
 
         # Log
         validation_data = ValidationMetricData(
-            source_model_loss=torch.tensor(losses),
-            source_model_loss_with_reconstruction=torch.tensor(losses_with_reconstruction),
-            source_model_loss_with_zero_ablation=torch.tensor(losses_with_zero_ablation),
+            source_model_loss=losses,
+            source_model_loss_with_reconstruction=losses_with_reconstruction,
+            source_model_loss_with_zero_ablation=losses_with_zero_ablation,
         )
         for metric in self.metrics.validation_metrics:
-            calculated = metric.calculate(validation_data)
+            log = {}
+            for metric_result in metric.calculate(validation_data):
+                log.update(metric_result.wandb_log)
             if wandb.run is not None:
-                wandb.log(data=calculated, commit=False)
+                wandb.log(log, commit=False)
 
     @final
     def save_checkpoint(self, *, is_final: bool = False) -> Path:
@@ -352,34 +398,26 @@ class Pipeline:
         Returns:
             Path to the saved checkpoint.
         """
-        # Create the name
         name: str = f"{self.run_name}_{'final' if is_final else self.total_activations_trained_on}"
-        safe_name = quote_plus(name, safe="_")
 
-        # Save locally
-        self.checkpoint_directory.mkdir(parents=True, exist_ok=True)
-        file_path: Path = self.checkpoint_directory / f"{safe_name}.pt"
-        torch.save(
-            self.autoencoder.state_dict(),
-            file_path,
-        )
-
-        # Upload to wandb
+        # Wandb
         if wandb.run is not None:
-            artifact = wandb.Artifact(safe_name, type="model")
-            artifact.add_file(str(file_path))
-            wandb.log_artifact(artifact)
+            self.autoencoder.save_to_wandb(name)
 
-        return file_path
+        # Local
+        local_path = self.checkpoint_directory / f"{name}.pt"
+        self.autoencoder.save(local_path)
+        return local_path
 
+    @validate_call
     def run_pipeline(
         self,
-        train_batch_size: int,
-        max_store_size: int,
-        max_activations: int,
-        validation_number_activations: int = 1024,
-        validate_frequency: int | None = None,
-        checkpoint_frequency: int | None = None,
+        train_batch_size: PositiveInt,
+        max_store_size: PositiveInt,
+        max_activations: PositiveInt,
+        validation_n_activations: PositiveInt = 1024,
+        validate_frequency: PositiveInt | None = None,
+        checkpoint_frequency: PositiveInt | None = None,
     ) -> None:
         """Run the full training pipeline.
 
@@ -388,7 +426,7 @@ class Pipeline:
             max_store_size: Maximum size of the activation store.
             max_activations: Maximum total number of activations to train on (the original paper
                 used 8bn, although others have had success with 100m+).
-            validation_number_activations: Number of activations to use for validation.
+            validation_n_activations: Number of activations to use for validation.
             validate_frequency: Frequency at which to get validation metrics.
             checkpoint_frequency: Frequency at which to save a checkpoint.
         """
@@ -412,9 +450,9 @@ class Pipeline:
                 activation_store: TensorActivationStore = self.generate_activations(store_size)
 
                 # Update the counters
-                num_activation_vectors_in_store = len(activation_store)
-                last_validated += num_activation_vectors_in_store
-                last_checkpoint += num_activation_vectors_in_store
+                n_activation_vectors_in_store = len(activation_store)
+                last_validated += n_activation_vectors_in_store
+                last_checkpoint += n_activation_vectors_in_store
 
                 # Train
                 progress_bar.set_postfix({"stage": "train"})
@@ -438,9 +476,10 @@ class Pipeline:
                         if wandb.run is not None:
                             wandb.log(
                                 {
-                                    "resample/dead_neurons": len(
-                                        parameter_updates.dead_neuron_indices
-                                    )
+                                    "resample/dead_neurons": [
+                                        len(update.dead_neuron_indices)
+                                        for update in parameter_updates
+                                    ]
                                 },
                                 commit=False,
                             )
@@ -451,7 +490,7 @@ class Pipeline:
                 # Get validation metrics (if needed)
                 progress_bar.set_postfix({"stage": "validate"})
                 if validate_frequency is not None and last_validated >= validate_frequency:
-                    self.validate_sae(validation_number_activations)
+                    self.validate_sae(validation_n_activations)
                     last_validated = 0
 
                 # Checkpoint (if needed)
@@ -465,43 +504,3 @@ class Pipeline:
 
         # Save the final checkpoint
         self.save_checkpoint(is_final=True)
-
-    @staticmethod
-    def stateful_dataloader_iterable(
-        dataloader: DataLoader[TorchTokenizedPrompts],
-    ) -> Iterable[TorchTokenizedPrompts]:
-        """Create a stateful dataloader iterable.
-
-        Create an iterable that maintains it's position in the dataloader between loops.
-
-        Examples:
-            Without this, when iterating over a DataLoader with 2 loops, each loop get the same data
-            (assuming shuffle is turned off). That is to say, the second loop won't maintain the
-            position from where the first loop left off.
-
-            >>> from datasets import Dataset
-            >>> from torch.utils.data import DataLoader
-            >>> def gen():
-            ...     yield {"int": 0}
-            ...     yield {"int": 1}
-            >>> data = DataLoader(Dataset.from_generator(gen))
-            >>> next(iter(data))["int"], next(iter(data))["int"]
-            (tensor([0]), tensor([0]))
-
-            By contrast if you create a stateful iterable from the dataloader, each loop will get
-            different data.
-
-            >>> iterator = Pipeline.stateful_dataloader_iterable(data)
-            >>> next(iterator)["int"], next(iterator)["int"]
-            (tensor([0]), tensor([1]))
-
-        Args:
-            dataloader: PyTorch DataLoader.
-
-        Returns:
-            Stateful iterable over the data in the dataloader.
-
-        Yields:
-            Data from the dataloader.
-        """
-        yield from dataloader
