@@ -174,6 +174,9 @@ class SourceDataHyperparameters(NestedParameter):
     context_size: Parameter[int] = field(default=Parameter(DEFAULT_SOURCE_CONTEXT_SIZE))
     """Context size."""
 
+    dataset_column_name: Parameter[str] | None = field(default=Parameter(value="input_ids"))
+    """Dataset column name."""
+
     dataset_dir: Parameter[str] | None = field(default=None)
     """Dataset directory (within the HF dataset)"""
 
@@ -211,6 +214,7 @@ class SourceDataRuntimeHyperparameters(TypedDict):
     """Source data runtime hyperparameters."""
 
     context_size: int
+    dataset_column_name: str
     dataset_dir: str | None
     dataset_files: list[str] | None
     dataset_path: str
@@ -226,12 +230,9 @@ class SourceModelHyperparameters(NestedParameter):
     name: Parameter[str]
     """Source model name."""
 
-    hook_site: Parameter[str]
+    cache_names: Parameter[list[str]]
     """Source model hook site."""
 
-    hook_layer: Parameter[int]
-    """Source model hook point layer."""
-
     hook_dimension: Parameter[int]
     """Source model hook point dimension."""
 
@@ -243,8 +244,7 @@ class SourceModelRuntimeHyperparameters(TypedDict):
     """Source model runtime hyperparameters."""
 
     name: str
-    hook_site: str
-    hook_layer: int
+    cache_names: list[str]
     hook_dimension: int
     dtype: str
 
@@ -280,9 +280,8 @@ class PipelineHyperparameters(NestedParameter):
     )
     """Validation frequency."""
 
-    validation_number_activations: Parameter[int] = field(
-        # Default to a single batch of source data prompts
-        default=Parameter(DEFAULT_BATCH_SIZE * DEFAULT_SOURCE_CONTEXT_SIZE * 16)
+    validation_n_activations: Parameter[int] = field(
+        default=Parameter(DEFAULT_SOURCE_BATCH_SIZE * DEFAULT_SOURCE_CONTEXT_SIZE * 2)
     )
     """Number of activations to use for validation."""
 
@@ -297,7 +296,7 @@ class PipelineRuntimeHyperparameters(TypedDict):
     max_activations: int
     checkpoint_frequency: int
     validation_frequency: int
-    validation_number_activations: int
+    validation_n_activations: int
 
 
 @dataclass
